/* jasen.cpp */
/****************************************************************************/
/*
** Jsen-luokan toteutus
**
**  Tekijt:          Vesa Lappalainen
**                    Ohjelmointikurssi 1991
**  Tehty:            20.12.1991
**  Muutettu:         09.12.1995
**  Mit muutettu:    Muutettu C++ luokaksi
**  Muutettu:         10.12.1995
**  Mit muutettu:
**  Muutettu:         20.01.1997
**  Mit muutettu:    Listty tunnusnumeron ksittely
**
** Public-metodeja:
** ================
**
**  cJasen()
**   - alustaa jsenen tyhjksi
**
**  void alusta();
**   - alustaa jsenen tyhjksi
**
**  int  setAsString(string &st);
**   - alustaa jsenen merkkijonolla 1|Ankka Aku|0....|..
**
**  string getAsString();
**   - palauttaa jsenen merkkijonona 1|Ankka Aku|0....|..
**
**  cJasen(const cJasen &jasen)
**   - alustaa jsenen toisen jsenen tiedoilla
**
**  const cJasen &operator=(const cJasen &jasen)
**   - sijoitaa jsen tiedot toiseen jseneen
**
**  const cJasen &sijoita(const cJasen &jasen);
**   - sijoitaa jsen tiedot toiseen jseneen
**
**  void vastaa_aku_ankka();
**   - tytt jsenen Aku Ankan tiedoilla (testitarkoituksiin)
**
**  int operator==(const cJasen &jasen) const;
**   - tutkii onko kaksi jsent samat ( 0 = ei, 1 = juu )
**
**  void tulosta(ostream &os) const;
**   - tulostaa jsenen valittuun tietovirtaan (tiedostoon) "tarra"-muodossa
**
**  ostream &operator<<(ostream &os,const cJasen &jasen);
**   - tulostaa jsenen rivimuodossa tietovirtaan
**
**  istream &operator>>(istream &is,cJasen &jasen);
**   - ottaa rivimuodossa olevan jsenen tietovirrasta
**  const string kentta_jonoksi(int k) const
**   - paluttaa k:n kentn merkkijonona
**
**  int sijoita(int k,const string &st)
**   - sijoittaa merkkijonon k:nteen kenttn
**
**  string kysymys(int k) const
**   - paluttaa k:ta kentt vastaavan kysymyksen
**
**  int kenttia() const
**   - palauttaa jsenen kenttien lukumrn
**
**  int tyhjenna_omat()
**   - tyhjent jsenen ne tiedot, jotka ovat omia uudelle
**     jsenelle
**
** Jsenen rivimuoto:
** ==================
**  1|Ankka Aku         |010245-123U|Ankkakuja 6 |12345|...
**
*/

#include <iostream>
#include <strstream>
#include <cstdlib>
#include <string>
//using namespace std;
#include "jasen.h"
#include "mjonotpp.h"


//============================================================================
int cJasen::seuraava_nro = 1;

static cVirheKentta VirheKentta("VIRHE");

//============================================================================
// Apufunktioita muunnoksiin:
//============================================================================
inline void muunna_jono(const string &st, string &s)
{
  s = st;
}

void muunna_jono(const string &st, int &i)
{
  i = -1;
  sscanf(st.c_str(),"%d",&i);
}

void muunna_jono(const string &st, double &d)
{
  d = -1;
  sscanf(st.c_str(),"%lf",&d);
}



//============================================================================
// Jsen-luokan public-metodeja:
//============================================================================

//----------------------------------------------------------------------------
int cJasen::alusta()
{
  for (int k=0; k<kenttia(); k++)
    sijoita(k,"");
  return 0;
}

//----------------------------------------------------------------------------
int cJasen::tyhjenna_omat()
{
  nimi        = "";
  hetu        = "";
  autopuhelin = "";
  lisatietoja = "";
  return 0;
}

char cJasen::erotin = '|';
string cJasen::erottimet = "|";

//----------------------------------------------------------------------------
int cJasen::setAsString(string &jono)
{
  string pala;
  for (int k=0; k<kenttia(); k++)  {
    pala = erota(jono,erottimet);
    sijoita(k,pala);
  }

  if ( tunnus_nro >= seuraava_nro ) seuraava_nro = tunnus_nro + 1;
  return 0;
}

//----------------------------------------------------------------------------
void cJasen::vastaa_aku_ankka()
{
  char apuhetu[50];
  sprintf(apuhetu,"%02d%02d%02d-%03d%c",
          rand()%31+1,rand()%12+1,rand()%91+1,rand()%1000,rand()%25+'A');
  apuhetu[11]=0;

  nimi           = "Ankka Aku";
  hetu           = apuhetu;
  katuosoite     = "Ankkakuja 6";
  postinumero    = "12345";
  postiosoite    = "ANKKALINNA";
  kotipuhelin    = "12-1234";
  tyopuhelin     = "";
  autopuhelin    = "";
  liittymisvuosi = 1996;
  jmaksu         = 50.00;
  maksu          = 30.00;
  lisatietoja    = "Velkaa Roopelle";
}


//----------------------------------------------------------------------------
void cJasen::tulosta(std::ostream &os) const
{
  os << "#"  << tunnus_nro << "\n";
  os << "  " << nimi << "  " << hetu << "\n";
  os << "  " << katuosoite << "  " <<
                postinumero << " " << postiosoite << "\n";
  os << "  k: " << kotipuhelin <<
         " t: " << tyopuhelin  <<
         " a: " << autopuhelin << "\n";
  os << "  Liittynyt "  << liittymisvuosi << "."
        "  Jsenmaksu " << jmaksu << " mk."
        "  Maksettu "   << maksu  << " mk." << "\n";
  os << "  " << lisatietoja << std::endl;
}

//----------------------------------------------------------------------------
const cJasen &cJasen::sijoita(const cJasen &jasen)
// Sijoitetaan kentt kerrallaan.
{
  int kenttia=jasen.kenttia();

  for (int k=0; k<kenttia; k++) {
    kentta(k).sijoita(jasen[k]);
  }
  return *this;
}


//----------------------------------------------------------------------------
const cKentta &cJasen::rkentta(int k) const
{
  switch ( k ) {
    case  0: return tunnus_nro;
    case  1: return nimi;
    case  2: return hetu;
    case  3: return katuosoite;
    case  4: return postinumero;
    case  5: return postiosoite;
    case  6: return kotipuhelin;
    case  7: return tyopuhelin;
    case  8: return autopuhelin;
    case  9: return liittymisvuosi;
    case 10: return jmaksu;
    case 11: return maksu;
    case 12: return lisatietoja;
    default: return VirheKentta;
  }
}

//----------------------------------------------------------------------------
int cJasen::operator==(const cJasen &jasen) const
// Verrataan onko tiedot samat
{
  int kenttia=jasen.kenttia();

  for (int k=0; k<kenttia; k++) {
    if ( kentta(k).jonoksi() != jasen[k].jonoksi() ) return 0;
  }
  return 1;
}

//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
string cJasen::getAsString() const
{
  string st;
  for (int k=0; k<kenttia(); k++)
    st += kentta_jonoksi(k) + erotin;
  return st;
}


//----------------------------------------------------------------------------
std::ostream &operator<<(std::ostream &os,const cJasen &jasen)
{
  os << jasen.getAsString();
  return os;
}

//----------------------------------------------------------------------------
std::istream &operator>>(std::istream &is,cJasen &jasen)
{
  string jono;
  std::getline(is,jono);
  jasen.setAsString(jono);
  return is;
}


//----------------------------------------------------------------------------
string cJasen::kysymys(int k) const
{
  switch ( k ) {
    case  0: return "Tunnus nro";
    case  1: return "Jsenen nimi";
    case  2: return "Hetu";
    case  3: return "Katuosoite";
    case  4: return "Postinumero";
    case  5: return "Postiosoite";
    case  6: return "Kotipuhelin";
    case  7: return "Typuhelin";
    case  8: return "Autopuhelin";
    case  9: return "Liittymisvuosi";
    case 10: return "Jsenmaksu mk";
    case 11: return "Maksettu maksu mk";
    case 12: return "Listietoja";
    default: return "VIRHE!";
  }
}
